% This script runs the CPF-SAEM algorithm (conditional particle filter
% stochastic approximation EM) to identify a nonlinear state-space model.
% The algorithm is presented in,
%
%   F. Lindsten, "An efficient stochastic approximation EM algorithm
%   using conditional particle filters", Proceedings of the 38th
%   International Conference on Acoustics, Speech, and Signal Processing
%   (ICASSP), Vancouver, Canada, May 2013.
%
% The script generates a batch of data y_{1:T} from the the standard
% nonlinear time series model,
%
%   x_{t+1} = 0.5*x_t + 25*x_t/(1+x_t^2) + 8*cos(1.2*t) + v_t,
%   y_t = 0.05*x_t^2 + e_t,
%
% with v_t ~ N(0,q) and e_t ~ N(0,r). The process noise and measurement
% noise variances (q,r) are treated as unkown parameters and the maximum
% likelihood estimates are computed using CPF-SAEM. For comparison, a
% standard Monte Carlo EM algorithm with particle smoothing (PSEM) is also
% implemented.
%
%
%   Fredrik Lindsten
%   Linkping, 2013-05-20
%   lindsten@isy.liu.se

%% CPF-SAEM ===============================================================
% Set up some parameters
N1 = 15;                % Number of particles used in CPF-SAEM
N2 = 1500;              % Number of particles used in PSEM
T = 100;                % Length of data record
numIter = 500;          % Number of iterations in EM algorithms
plotOn  = 0;            % Plot intermediate results?
kappa = 1;              % Constant used to compute SA step length (see below)

% Generate data
q0 = 0.1;  % True process noise variance
r0 = 1; % True measurement noise variance
[x0,y0] = generate_data(T,q0,r0);

% Initialization for the parameters
qinit = 1;
rinit = 0.1;

% SA step length
gamma = zeros(1,numIter);
gamma(1:2) = 1;
gamma(3:99) = 0.98;
gamma(100:end) = 0.98*(((0:numIter-100)+kappa)/kappa).^(-0.7);

% Run the algorithms
fprintf('Running CPF-SAEM (N=%i). Progress: ',N1); tic;
[q1, r1] = cpf_saem(numIter, y0, N1, gamma, qinit, rinit, q0, r0, plotOn);
timeelapsed = toc;
fprintf(' Elapsed time: %2.2f sec.\n',timeelapsed);

fprintf('Running PSEM (N=%i). Progress: ',N2); tic;
[q2, r2] = psem(numIter, y0, N2, qinit, rinit, q0, r0, plotOn);
timeelapsed = toc;
fprintf(' Elapsed time: %2.2f sec.\n',timeelapsed);tic;

%% Plot the results

figure(1); clf(1);
subplot(2,1,1);
loglog([1, numIter],q0*[1 1],'k-'); hold on;
loglog(q1,'b-');
loglog(q2,'r-'); hold off;
legend('True','CPF-SAEM','PSEM');
title('Estimates of q')

subplot(2,1,2)
loglog([1, numIter],r0*[1 1],'k-'); hold on;
loglog(r1,'b-');
loglog(r2,'r-'); hold off;
legend('True','CPF-SAEM','PSEM');
title('Estimates of r')
